import React, { ReactElement } from 'react';
import styles from './buttons.module.scss';
import Icon, { Sizes } from '../icon';
import { forwardRef } from 'react';

type ButtonProps = {
  type?: 'primary' | 'secondary' | 'danger';
  variant?: 'solid' | 'outline' | 'ghost';
  size?: 'xs' | 'sm' | 'md' | 'lg';
  label?: string;
  rounded?: boolean;
  disabled?: boolean;
  block?: boolean;
  onClick: (e: any) => void;
  iconComponent?: React.ReactElement;
  iconPosition?: 'leading' | 'trailing';
  ariaLabel?: string;
  children?: React.ReactNode;
  additionalClasses?: string;
};

export const Button = React.forwardRef<HTMLButtonElement, ButtonProps & React.HTMLAttributes<HTMLButtonElement>>(
  (
    {
      type = 'secondary',
      variant = 'solid',
      label,
      size = 'md',
      rounded = false,
      disabled = false,
      onClick,
      block = false,
      iconComponent,
      iconPosition = 'leading',
      ariaLabel,
      additionalClasses,
      children,
      ...props
    },
    forwardRef,
  ) => {
    let typeClass = '';
    let variantClass = '';
    let sizeClass = '';
    const blockClass = block ? styles['btn-block'] : '';
    const roundedClass = rounded ? styles['btn-rounded'] : '';

    switch (type) {
      case 'primary':
        typeClass = styles['btn-primary'];
        break;
      case 'secondary':
        typeClass = styles['btn-secondary'];
        break;
      case 'danger':
        typeClass = styles['btn-danger'];
        break;
      default:
        return null;
    }

    switch (variant) {
      case 'solid':
        variantClass = styles['btn-solid'];
        break;
      case 'outline':
        variantClass = styles['btn-outline'];
        break;
      case 'ghost':
        variantClass = styles['btn-ghost'];
        break;
      default:
        return null;
    }

    let iconSize: Sizes = 24;

    switch (size) {
      case 'xs':
        sizeClass = styles['btn-xs'];
        iconSize = 16;
        break;
      case 'sm':
        sizeClass = styles['btn-sm'];
        iconSize = 20;
        break;
      case 'md':
        sizeClass = styles['btn-md'];
        iconSize = 24;
        break;
      case 'lg':
        sizeClass = styles['btn-lg'];
        iconSize = 28;
        break;
      default:
        return null;
    }

    const icon = iconComponent ? <Icon component={iconComponent} size={iconSize} /> : null;

    const iconOnlyClass = iconComponent && !label && !children ? styles['btn-icon-only'] : '';

    return (
      <button
        className={`${styles.btn} ${typeClass} ${variantClass} ${sizeClass} ${blockClass} ${roundedClass} ${iconOnlyClass} ${additionalClasses}`}
        onClick={onClick}
        disabled={disabled}
        aria-label={ariaLabel}
        ref={forwardRef}
        {...props}
      >
        {iconPosition === 'leading' && icon}
        {label && <span>{label}</span>}
        {children && <span>{children}</span>}
        {iconPosition === 'trailing' && icon}
      </button>
    );
  },
);

type ButtonGroupProps = {
  children: React.ReactNode;
};

export function ButtonGroup({ children }: ButtonGroupProps) {
  return <div>{children}</div>;
}

type ButtonItemProps = {
  icon: React.ReactNode;
  onClick: () => void;
};

export function ButtonGroupItem({ icon, onClick }: ButtonItemProps) {
  return (
    <div onClick={onClick} className="rounded-sm bg-secondary-50 p-2">
      <span>{icon}</span>
    </div>
  );
}
