import { DimensionType } from '@graphpolaris/shared/lib/schema';
import { GraphMetaData, DimensionStatistics } from './statistics.types';
import { CategoricalStats, NumericalStats, SpatialStats, TemporalStats } from './statistics.types';

function extractCategoricalStats(data: any[]): CategoricalStats {
  const uniqueItems = new Set<string>(data.map((item) => String(item).slice(0, 50)));
  const uniqueItemCount = uniqueItems.size;

  // Calculate mode
  const frequencyMap: { [key: string]: number } = {};
  data.forEach((item) => {
    const strItem = String(item).slice(0, 100);
    frequencyMap[strItem] = (frequencyMap[strItem] || 0) + 1;
  });
  const mode = Object.keys(frequencyMap).reduce((a, b) => (frequencyMap[a] > frequencyMap[b] ? a : b), '');

  // Calculate entropy
  const totalItems = data.length;
  const entropy = -Object.values(frequencyMap).reduce((acc, frequency) => {
    const probability = frequency / totalItems;
    if (probability === 0) return acc;
    return acc + probability * Math.log2(probability);
  }, 0);

  // Calculate Gini Index
  const giniIndex =
    1 -
    Object.values(frequencyMap).reduce((acc, frequency) => {
      const probability = frequency / totalItems;
      if (probability === 0) return acc;
      return acc + Math.pow(probability, 2);
    }, 0);

  return {
    uniqueItems: uniqueItemCount,
    mode,
    entropy,
    giniIndex,
    values: Array.from(uniqueItems),
  };
}

function extractNumericalStats(data: number[]): NumericalStats {
  const n = data.length;
  if (n === 0) {
    return {
      min: 0,
      max: 0,
      average: 0,
      variance: 0,
      std: 0,
      kurtosis: 0,
    };
  }

  const min = Math.min(...data);
  const max = Math.max(...data);
  const sum = data.reduce((acc, curr) => acc + curr, 0);
  const average = sum / n;

  let variance = 0;
  for (const value of data) {
    variance += Math.pow(value - average, 2);
  }
  variance /= n;
  const std = Math.sqrt(variance);

  let kurtosis = 0;
  if (std !== 0) {
    const fourthMoment = data.reduce((acc, curr) => acc + Math.pow(curr - average, 4), 0) / n;
    kurtosis = fourthMoment / Math.pow(std, 4) - 3;
  }

  return {
    min,
    max,
    average,
    variance,
    std,
    kurtosis: Number.isFinite(kurtosis) ? kurtosis : 0,
  };
}

function extractTemporalStats(data: Date[]): TemporalStats {
  const sortedDates = data.slice().sort((a, b) => a.getTime() - b.getTime());
  const minDate = sortedDates[0];
  const maxDate = sortedDates[sortedDates.length - 1];
  const range = maxDate.getTime() - minDate.getTime();

  return {
    minDate,
    maxDate,
    range,
    minTimestamp: minDate.getTime(),
    maxTimestamp: maxDate.getTime(),
  };
}

function extractSpatialStats(data: [number, number][]): SpatialStats {
  const totalPoints = data.length;
  const sumCoordinates = data.reduce(([x1, y1], [x2, y2]) => [x1 + x2, y1 + y2], [0, 0]);
  const middlePoint: [number, number] = [sumCoordinates[0] / totalPoints, sumCoordinates[1] / totalPoints];

  const minX = Math.min(...data.map(([x, _]) => x));
  const minY = Math.min(...data.map(([_, y]) => y));
  const maxX = Math.max(...data.map(([x, _]) => x));
  const maxY = Math.max(...data.map(([_, y]) => y));

  const boundingBox: [[number, number], [number, number]] = [
    [minX, minY],
    [maxX, maxY],
  ];

  return {
    middlePoint,
    minX,
    minY,
    maxX,
    maxY,
    boundingBox,
  };
}

export const extractStatistics = (summaryGraph: GraphMetaData): GraphMetaData => {
  const graph = summaryGraph;

  Object.entries(graph.nodes.types).forEach(([label, node]) => {
    Object.entries(node.attributes).forEach(([id, object]) => {
      const { values, dimension } = object;
      const statistics = calculateStatistics(values || [], dimension);
      graph.nodes.types[label].attributes[id] = { dimension, ...statistics };
    });
  });

  Object.entries(graph.edges.types).forEach(([label, edge]) => {
    Object.entries(edge.attributes).forEach(([id, object]) => {
      const { values, dimension } = object;
      const statistics = calculateStatistics(values || [], dimension);
      graph.edges.types[label].attributes[id] = { dimension, ...statistics };
    });
  });

  console.debug('Graph query statistics', summaryGraph);
  return graph;
};

export const calculateStatistics = (item: any[], dimensionType: DimensionType) => {
  let statistics: DimensionStatistics | undefined;

  switch (dimensionType) {
    case 'categorical':
      statistics = extractCategoricalStats(item);
      break;
    case 'numerical':
      statistics = extractNumericalStats(item);
      break;
    case 'spatial':
      statistics = extractSpatialStats(item);
      break;
    case 'temporal':
      statistics = extractTemporalStats(item);
      break;
    default:
      statistics = extractCategoricalStats(item);
  }

  return statistics;
};
