import React, { useState, useEffect } from 'react';
import { VISComponentType } from './types';
import { updateVisualization } from '../data-access/store/visualizationSlice';
import { useAppDispatch, useGraphQueryResult, useGraphQueryResultMeta, useML, useSchemaGraph, useVisualization } from '../data-access';
import { FormActions, FormBody, FormCard, FormDiv, FormHBar, FormTitle } from '../components';

export const Visualizations: Record<string, Function> = {
  TableVis: () => import('./visualizations/tablevis/tablevis'),
  PaohVis: () => import('./visualizations/paohvis/paohvis'),
  RawJSONVis: () => import('./visualizations/rawjsonvis/rawjsonvis'),
  NodeLinkVis: () => import('./visualizations/nodelinkvis/nodelinkvis'),
  // MapVis: () => import('./visualizations/mapvis/mapvis'),
  MatrixVis: () => import('./visualizations/matrixvis/matrixvis'),
};

export const useVisualizationManager = () => {
  const dispatch = useAppDispatch();
  const ml = useML();
  const schema = useSchemaGraph();
  const graphQueryResult = useGraphQueryResult();
  const meta = useGraphQueryResultMeta();
  const { active, visualizations } = useVisualization();

  const [configuration, setConfiguration] = useState<any>();
  const [visualization, setVisualization] = useState<VISComponentType>();
  const [settingsOpen, setSettingsOpen] = useState<boolean>(false);
  const [hoverItem, setHoverItem] = useState<any>();
  const [selected, setSelected] = useState<any>();

  useEffect(() => {
    loadVisualization();
  }, [active]);

  const loadVisualization = async () => {
    if (active && Visualizations[active]) {
      const componentModule = await Visualizations[active]();
      const component = componentModule.default;

      if (!(active in Object.keys(visualizations))) {
        // Visualization doesn't yet exist so add its configuration
        const configuration = component.configuration;
        updateSettings(configuration);
      }

      setConfiguration(visualizations[active]);
      setVisualization(component);
    }
  };

  const handleHover = (item: any) => {
    setHoverItem(item);
  };

  const handleSelect = (item: any) => {
    setSelected(item);
  };

  const updateSettings = (newSettings: any) => {
    if (active) {
      const updatedSettings = { ...configuration, ...newSettings };
      setConfiguration(updatedSettings);
      dispatch(updateVisualization({ id: active, settings: updatedSettings }));
    }
  };

  const openSettingsMenu = () => {
    setSettingsOpen(!settingsOpen);
  };

  const renderSettings = () => {
    return (
      visualization?.settings &&
      configuration &&
      settingsOpen && (
        <FormDiv>
          <FormCard>
            <FormBody
              onSubmit={(e) => {
                e.preventDefault();
                setSettingsOpen(false);
              }}
            >
              <FormTitle title="Visualization settings" onClose={() => {}} />
              <FormHBar />
              <visualization.settings configuration={configuration} graph={meta} updateSettings={updateSettings} />
              <FormHBar />
              <FormActions
                onClose={() => {
                  setSettingsOpen(false);
                }}
              />
            </FormBody>
          </FormCard>
        </FormDiv>
      )
    );
  };

  const renderComponent = () => {
    return (
      visualization?.component &&
      configuration && (
        <visualization.component
          data={graphQueryResult}
          schema={schema}
          ml={ml}
          configuration={configuration}
          dispatch={dispatch}
          handleHover={handleHover}
          handleSelect={handleSelect}
        />
      )
    );
  };

  return {
    active,
    renderComponent,
    renderSettings,
    openSettingsMenu,
  };
};
